#ifndef MPC_PLANNER_STRUCT_H
#define MPC_PLANNER_STRUCT_H

#include "genPosStruct.h"
#include <MPC/mpc-gen.hpp>

/***********************************************************************/
/* Robotic system type */

#define MPC_PLANNER_UNDEFINED_SYS 0
#define MPC_PLANNER_MANA_SYS 1

#define PL_ROBOTIC_SYS_T int


/***********************************************************************/
/* 2D path tab max size */
#define PL_PLAN_MAXSIZE 5000
#define GEN_PLAN_MAXSIZE 100

#define cfg_status int
#define CFG_FREE 0
#define CFG_COL 1

/* Struct containing a 2D path */
typedef struct PL_PATH_XY
{
  GENPOS_CART_CONFIG tab[PL_PLAN_MAXSIZE];
  int nbPos;
/*   float corridorWidth; */
} PL_PATH_XY; 

/* Struct containing a 2D path */
typedef struct GEN_PATH_XY
{
  GENPOS_CART_CONFIG tab[GEN_PLAN_MAXSIZE];
  cfg_status status[GEN_PLAN_MAXSIZE];
  int nbPos;
} GEN_PATH_XY; 


/***********************************************************************/
/* Struct containing Mana system constraints */

typedef struct PL_MANA_SYS_CSTT 
{
  double length;                /* length of the Mana base */
  double width;                 /* length of the Mana base */
  double vMax;                  /* maximal speed */
  double wMax;                  /* maximal angular speed */
  double aMax;                  /* maximal acceleration */
  double uMax;                  /* maximal angular acceleration */
} PL_MANA_SYS_CSTT;

/***********************************************************************/
/* Struct containing Mana state-time */

typedef struct PL_MANA_STATET {
  double x;                     /* X position */
  double y;                     /* Y position */
  double th;                    /* TH heading */
  double v;                     /* V linear speed */
  double w;                     /* W angular speed */
  double t;                     /* T time */
} PL_MANA_STATET;

/***********************************************************************/

/* Struct containing a Mana state trajectory */
typedef struct PL_MANA_STRAJ 
{
  PL_MANA_STATET tab[PL_PLAN_MAXSIZE];
  int nbPos;
} PL_MANA_STRAJ;


/***********************************************************************/
/**************************** Requests I/O *****************************/
/***********************************************************************/

/***********************************************************************/
/* Initialization parameters */

typedef struct PL_INIT_I
{
/*   char robot[256]; */
  char pomPoster[256];
  char dtmPoster[256];
  char ctrlPoster[256];
} PL_INIT_I;

/***********************************************************************/
/* SetMode parameters */

#define UNDEFINED_PL_MODE 0
#define SIMPLE_PL_MODE 1

#define PL_MODE int

#define UNDEFINED_PL_OUTPUT 0
#define PATH_PL_OUTPUT 1
#define STRAJ_PL_OUTPUT 2

#define PL_OUTPUT int

typedef struct PL_SETMODE_I
{
  char mode[256];
  char output[256];
} PL_SETMODE_I;

/***********************************************************************/
/* PlanTo parameters */

typedef struct PL_PLANTO_I 
{
  double goalX;
  double goalY;
} PL_PLANTO_I;

/***********************************************************************/
/* PlanSimplePath parameters */

typedef struct PL_PLANSIMPLEPATH_I
{
  GENPOS_CART_CONFIG initPos;
  GENPOS_CART_CONFIG goalPos;
} PL_PLANSIMPLEPATH_I;

/***********************************************************************/
/* PlanSimplePath parameters */

typedef struct PL_PLANSIMPLESTRAJ_I
{
  GENPOS_CART_CONFIG initPos;
  GENPOS_CART_CONFIG goalPos;
} PL_PLANSIMPLESTRAJ_I;

/***********************************************************************/
/* SetManaSys parameters */

typedef struct PL_SETMANASYS_I
{
  double body_length;
  double body_width;
  double body_z;
  double ground_clearance;
  double rollMax;
  double pitchMax;
  double vmin;
  double vmax;
  double accel;
  double decel;
  double wmax;
} PL_SETMANASYS_I;


/***********************************************************************/
/* StartPathRecording parameters */

typedef struct PATHRECORDING_I
{
  float minDistBtwnPos;
} PATHRECORDING_I;


/***********************************************************************/
/* StartTrajRecording parameters */

typedef struct TRAJRECORDING_I
{
  float dt;
} TRAJRECORDING_I;


/***********************************************************************/
/*  parameters */

typedef struct PVD_I
{
  float coefVel;
  float coefAcc;
} PVD_I;

/***********************************************************************/
/*  parameters */

typedef struct SHIFT_XY_I
{
  float dx;
  float dy;
} SHIFT_XY_I;

/***********************************************************************/
/*  parameters */

typedef struct FILENAME_I
{
  char filename[256];
} FILENAME_I;

/***********************************************************************/
/*  parameters */

typedef struct LOADPATH_I
{
  char filename[256];
  int inverted;
} LOADPATH_I;

#endif /* MPC_PLANNER_STRUCT_H */
